@@ -42,12 +42,15 @@
 /* Must be before all other gnome includes!! */
 #include "gnome-i18nP.h"
 
+#include "gnome-marshal.h"
 #include "gnome-color-picker.h"
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtkbutton.h>
 
 #include <libgnomeuiP.h>
 
+#include "libgnomeui-access.h"
+
 /* These are the dimensions of the color sample in the color picker */
 #define COLOR_PICKER_WIDTH  20
 #define COLOR_PICKER_HEIGHT 12
@@ -92,8 +95,6 @@ enum {
 	LAST_SIGNAL
 };
 
-static void gnome_color_picker_class_init (GnomeColorPickerClass *class);
-static void gnome_color_picker_instance_init (GnomeColorPicker      *cp);
 static void gnome_color_picker_destroy    (GtkObject             *object);
 static void gnome_color_picker_finalize   (GObject               *object);
 static void gnome_color_picker_clicked    (GtkButton             *button);
@@ -129,7 +130,7 @@ static guint color_picker_signals[LAST_SIGNAL] = { 0 };
 static GtkTargetEntry drop_types[] = { { "application/x-color", 0, 0 } };
 
 GNOME_CLASS_BOILERPLATE (GnomeColorPicker, gnome_color_picker,
-			 GtkButton, gtk_button, GTK_TYPE_BUTTON)
+			 GtkButton, GTK_TYPE_BUTTON)
 
 static void
 gnome_color_picker_class_init (GnomeColorPickerClass *class)
@@ -145,16 +146,17 @@ gnome_color_picker_class_init (GnomeColorPickerClass *class)
 	widget_class = (GtkWidgetClass *) class;
 
 	color_picker_signals[COLOR_SET] =
-		gtk_signal_new ("color_set",
-				GTK_RUN_FIRST,
-				GTK_CLASS_TYPE (object_class),
-				GTK_SIGNAL_OFFSET (GnomeColorPickerClass, color_set),
-				gnome_marshal_VOID__UINT_UINT_UINT_UINT,
-				GTK_TYPE_NONE, 4,
-				GTK_TYPE_UINT,
-				GTK_TYPE_UINT,
-				GTK_TYPE_UINT,
-				GTK_TYPE_UINT);
+		g_signal_new ("color_set",
+			      G_TYPE_FROM_CLASS (gobject_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GnomeColorPickerClass, color_set),
+			      NULL, NULL,
+			      _gnome_marshal_VOID__UINT_UINT_UINT_UINT,
+			      G_TYPE_NONE, 4,
+			      G_TYPE_UINT,
+			      G_TYPE_UINT,
+			      G_TYPE_UINT,
+			      G_TYPE_UINT);
 
 	gobject_class->get_property = gnome_color_picker_get_property;
 	gobject_class->set_property = gnome_color_picker_set_property;
@@ -162,43 +164,50 @@ gnome_color_picker_class_init (GnomeColorPickerClass *class)
         g_object_class_install_property
                 (gobject_class,
                  PROP_DITHER,
-                 g_param_spec_boolean ("dither", NULL, NULL,
+                 g_param_spec_boolean ("dither", NULL, 
+			               "Whether or not to dither to color",
 				       TRUE,
 				       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_USE_ALPHA,
-                 g_param_spec_boolean ("use_alpha", NULL, NULL,
+                 g_param_spec_boolean ("use_alpha", NULL, 
+			               "Whether or not to give the color an alpha value",
 				       FALSE,
 				       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_TITLE,
-                 g_param_spec_string ("title", NULL, NULL,
+                 g_param_spec_string ("title", NULL, 
+			              "The title to give to the color picker",
 				      NULL,
 				      (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_RED,
-                 g_param_spec_uint ("red", NULL, NULL,
+                 g_param_spec_uint ("red", NULL,
+			            "The amount of red in the chosen color",
 				    0, 255, 0,
 				    (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_GREEN,
-                 g_param_spec_uint ("green", NULL, NULL,
+                 g_param_spec_uint ("green", NULL, 
+			            "The amount of green in the chosen color",
 				    0, 255, 0,
 				    (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_BLUE,
-                 g_param_spec_uint ("blue", NULL, NULL,
+                 g_param_spec_uint ("blue", NULL, 
+			            "The amount of blue in the chosen color",
 				    0, 255, 0,
 				    (G_PARAM_READABLE | G_PARAM_WRITABLE)));
         g_object_class_install_property
                 (gobject_class,
                  PROP_ALPHA,
-                 g_param_spec_uint ("alpha", NULL, NULL,
+                 g_param_spec_uint ("alpha", NULL, 
+			            "The alpha value of the chosen color",
 				    0, 255, 0,
 				    (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 
@@ -285,9 +294,9 @@ render_dither (GnomeColorPicker *cp)
 		}
 	}
 	if (cp->_priv->drawing_area->window)
-		gdk_pixbuf_render_to_drawable (cp->_priv->pixbuf,
-					       cp->_priv->drawing_area->window,
+		gdk_draw_pixbuf (cp->_priv->drawing_area->window,
 					       cp->_priv->gc,
+					       cp->_priv->pixbuf,
 					       0, 0, 0, 0,
 					       COLOR_PICKER_WIDTH,
 					       COLOR_PICKER_HEIGHT,
@@ -337,12 +346,10 @@ expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer data)
 	GnomeColorPicker *cp;
 
 	cp = GNOME_COLOR_PICKER (data);
-	if (cp->_priv->gc == NULL)
-		cp->_priv->gc = gdk_gc_new (widget->window);
 
-	gdk_pixbuf_render_to_drawable (cp->_priv->pixbuf,
-				       widget->window,
+	gdk_draw_pixbuf (widget->window,
 				       cp->_priv->gc,
+				       cp->_priv->pixbuf,
 				       event->area.x,
 				       event->area.y,
 				       event->area.x,
@@ -359,14 +366,18 @@ gnome_color_picker_realize (GtkWidget *widget)
 {
 	GnomeColorPicker *cp = GNOME_COLOR_PICKER (widget);
 
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, realize, (widget));
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, realize, (widget));
+
+	if (cp->_priv->gc == NULL)
+		cp->_priv->gc = gdk_gc_new (widget->window);
 
 	render (cp);
 }
+
 static void
 gnome_color_picker_style_set (GtkWidget *widget, GtkStyle *previous_style)
 {
-	GNOME_CALL_PARENT_HANDLER (GTK_WIDGET_CLASS, style_set,
+	GNOME_CALL_PARENT (GTK_WIDGET_CLASS, style_set,
 				   (widget, previous_style));
 
 	if (GTK_WIDGET_REALIZED (widget))
@@ -398,7 +409,7 @@ drag_data_received (GtkWidget        *widget,
 	if (selection_data->length < 0)
 		return;
 
-	if ((selection_data->format != 16) || 
+	if ((selection_data->format != 16) ||
 	    (selection_data->length != 8)) {
 		g_warning (_("Received invalid color data\n"));
 		return;
@@ -409,9 +420,12 @@ drag_data_received (GtkWidget        *widget,
 
 	gnome_color_picker_set_i16(cpicker, dropped[0], dropped[1],
 				   dropped[2], dropped[3]);
+	g_signal_emit (cpicker, color_picker_signals[COLOR_SET], 0,
+		       dropped[0], dropped[1], dropped[2], dropped[3]);
+		
 }
 
-static void  
+static void
 drag_data_get  (GtkWidget          *widget,
 		GdkDragContext     *context,
 		GtkSelectionData   *selection_data,
@@ -439,10 +453,17 @@ gnome_color_picker_instance_init (GnomeColorPicker *cp)
 {
 	GtkWidget *alignment;
 	GtkWidget *frame;
-	/* Create the widgets */
 
+	/* Create the widgets */
 	cp->_priv = g_new0(GnomeColorPickerPrivate, 1);
 
+	/*
+	 * The application may very well override these.
+	 */
+	_add_atk_name_desc (GTK_WIDGET (cp),
+			    _("Color Selector"),
+			    _("Open a dialog to specify the color"));
+
 	alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
 	gtk_container_set_border_width (GTK_CONTAINER (alignment), COLOR_PICKER_PAD);
 	gtk_container_add (GTK_CONTAINER (cp), alignment);
@@ -453,14 +474,13 @@ gnome_color_picker_instance_init (GnomeColorPicker *cp)
 	gtk_container_add (GTK_CONTAINER (alignment), frame);
 	gtk_widget_show (frame);
 
-	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
+	gtk_widget_push_colormap (gdk_rgb_get_colormap ());
 
 	cp->_priv->drawing_area = gtk_drawing_area_new ();
 
-	gtk_drawing_area_size (GTK_DRAWING_AREA (cp->_priv->drawing_area), COLOR_PICKER_WIDTH, COLOR_PICKER_HEIGHT);
-	gtk_signal_connect (GTK_OBJECT (cp->_priv->drawing_area), "expose_event",
-			    (GtkSignalFunc) expose_event,
-			    cp);
+	gtk_widget_set_size_request (cp->_priv->drawing_area, COLOR_PICKER_WIDTH, COLOR_PICKER_HEIGHT);
+	g_signal_connect (cp->_priv->drawing_area, "expose_event",
+			  G_CALLBACK (expose_event), cp);
 	gtk_container_add (GTK_CONTAINER (frame), cp->_priv->drawing_area);
 	gtk_widget_show (cp->_priv->drawing_area);
 
@@ -493,12 +513,10 @@ gnome_color_picker_instance_init (GnomeColorPicker *cp)
 			     GDK_BUTTON1_MASK|GDK_BUTTON3_MASK,
 			     drop_types, 1,
 			     GDK_ACTION_COPY);
-	gtk_signal_connect (GTK_OBJECT (cp),
-			    "drag_data_received",
-			    GTK_SIGNAL_FUNC (drag_data_received), cp);
-	gtk_signal_connect (GTK_OBJECT (cp),
-			    "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get), cp);
+	g_signal_connect (cp, "drag_data_received",
+			  G_CALLBACK (drag_data_received), cp);
+	g_signal_connect (cp, "drag_data_get",
+			  G_CALLBACK (drag_data_get), cp);
 }
 
 static void
@@ -513,21 +531,15 @@ gnome_color_picker_destroy (GtkObject *object)
 
 	cp = GNOME_COLOR_PICKER (object);
 
-	if(cp->_priv->pixbuf)
-		gdk_pixbuf_unref (cp->_priv->pixbuf);
-	cp->_priv->pixbuf = NULL;
-	if(cp->_priv->gc)
-		gdk_gc_destroy (cp->_priv->gc);
+	if (cp->_priv->gc != NULL)
+		g_object_unref (G_OBJECT (cp->_priv->gc));
 	cp->_priv->gc = NULL;
 
-	if (cp->_priv->cs_dialog)
+	if (cp->_priv->cs_dialog != NULL)
 		gtk_widget_destroy (cp->_priv->cs_dialog);
 	cp->_priv->cs_dialog = NULL;
 
-	g_free (cp->_priv->title);
-	cp->_priv->title = NULL;
-
-	GNOME_CALL_PARENT_HANDLER (GTK_OBJECT_CLASS, destroy, (object));
+	GNOME_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
 }
 
 static void
@@ -540,10 +552,17 @@ gnome_color_picker_finalize (GObject *object)
 
 	cp = GNOME_COLOR_PICKER (object);
 
+	if (cp->_priv->pixbuf != NULL)
+		g_object_unref (G_OBJECT (cp->_priv->pixbuf));
+	cp->_priv->pixbuf = NULL;
+
+	g_free (cp->_priv->title);
+	cp->_priv->title = NULL;
+
 	g_free (cp->_priv);
 	cp->_priv = NULL;
 
-	GNOME_CALL_PARENT_HANDLER (G_OBJECT_CLASS, finalize, (object));
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 
@@ -562,7 +581,7 @@ gnome_color_picker_finalize (GObject *object)
 GtkWidget *
 gnome_color_picker_new (void)
 {
-	return GTK_WIDGET (gtk_type_new (GNOME_TYPE_COLOR_PICKER));
+	return g_object_new (GNOME_TYPE_COLOR_PICKER, NULL);
 }
 
 /* Callback used when the color selection dialog is destroyed */
@@ -583,28 +602,33 @@ static void
 cs_ok_clicked (GtkWidget *widget, gpointer data)
 {
 	GnomeColorPicker *cp;
-	gdouble color[4];
+	GtkColorSelection *cs;
+	GdkColor color;
+	guint16 alpha;
 	gushort r, g, b, a;
 
 	cp = GNOME_COLOR_PICKER (data);
 
-	gtk_color_selection_get_color (GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (cp->_priv->cs_dialog)->colorsel),
-				       color);
+	cs = GTK_COLOR_SELECTION (GTK_COLOR_SELECTION_DIALOG (cp->_priv->cs_dialog)->colorsel);
+
+	gtk_color_selection_get_current_color (cs, &color);
+	alpha = gtk_color_selection_get_current_alpha (cs);
+
 	gtk_widget_destroy (cp->_priv->cs_dialog);
 
-	cp->_priv->r = color[0];
-	cp->_priv->g = color[1];
-	cp->_priv->b = color[2];
-	cp->_priv->a = cp->_priv->use_alpha ? color[3] : 1.0;
+	cp->_priv->r = color.red / 65535.0;
+	cp->_priv->g = color.green / 65535.0;
+	cp->_priv->b = color.blue / 65535.0;
+	cp->_priv->a = cp->_priv->use_alpha ? (alpha / 65535.0) : 1.0;
 
 	render (cp);
-	gtk_widget_draw (cp->_priv->drawing_area, NULL);
+	gtk_widget_queue_draw (cp->_priv->drawing_area);
 
 	/* Notify the world that the color was set */
 
 	gnome_color_picker_get_i16 (cp, &r, &g, &b, &a);
-	gtk_signal_emit (GTK_OBJECT (cp), color_picker_signals[COLOR_SET],
-			 r, g, b, a);
+	g_signal_emit (cp, color_picker_signals[COLOR_SET], 0,
+		       r, g, b, a);
 }
 
 static int
@@ -622,7 +646,8 @@ gnome_color_picker_clicked (GtkButton *button)
 {
 	GnomeColorPicker *cp;
 	GtkColorSelectionDialog *csd;
-	gdouble color[4];
+	GdkColor color;
+	guint16 alpha;
 
 	g_return_if_fail (button != NULL);
 	g_return_if_fail (GNOME_IS_COLOR_PICKER (button));
@@ -640,27 +665,27 @@ gnome_color_picker_clicked (GtkButton *button)
                 GtkWidget *parent;
 
                 parent = gtk_widget_get_toplevel(GTK_WIDGET(cp));
-                
+
 		cp->_priv->cs_dialog = gtk_color_selection_dialog_new (cp->_priv->title);
 
                 if (parent)
                         gtk_window_set_transient_for(GTK_WINDOW(cp->_priv->cs_dialog),
                                                      GTK_WINDOW(parent));
-                
+
 		csd = GTK_COLOR_SELECTION_DIALOG (cp->_priv->cs_dialog);
-		gtk_signal_connect (GTK_OBJECT (cp->_priv->cs_dialog), "destroy",
-				    (GtkSignalFunc) cs_destroy,
-				    cp);
+		gtk_color_selection_set_has_palette
+		    (GTK_COLOR_SELECTION (csd->colorsel), TRUE);
+		g_signal_connect (cp->_priv->cs_dialog, "destroy",
+				  G_CALLBACK (cs_destroy), cp);
 
-		gtk_signal_connect (GTK_OBJECT (cp->_priv->cs_dialog), "key_press_event",
-				    (GtkSignalFunc) key_pressed, cp);
-		gtk_signal_connect (GTK_OBJECT (csd->ok_button), "clicked",
-				    (GtkSignalFunc) cs_ok_clicked,
-				    cp);
+		g_signal_connect (cp->_priv->cs_dialog, "key_press_event",
+				  G_CALLBACK (key_pressed), cp);
+		g_signal_connect (csd->ok_button, "clicked",
+				  G_CALLBACK (cs_ok_clicked), cp);
 
-		gtk_signal_connect_object (GTK_OBJECT (csd->cancel_button), "clicked",
-					   (GtkSignalFunc) gtk_widget_destroy,
-					   GTK_OBJECT(cp->_priv->cs_dialog));
+		g_signal_connect_swapped (csd->cancel_button, "clicked",
+					  G_CALLBACK (gtk_widget_destroy),
+					  cp->_priv->cs_dialog);
 
 		/* FIXME: do something about the help button */
 
@@ -673,14 +698,16 @@ gnome_color_picker_clicked (GtkButton *button)
 	gtk_color_selection_set_has_opacity_control (GTK_COLOR_SELECTION (csd->colorsel),
 						     cp->_priv->use_alpha);
 
-	color[0] = cp->_priv->r;
-	color[1] = cp->_priv->g;
-	color[2] = cp->_priv->b;
-	color[3] = cp->_priv->use_alpha ? cp->_priv->a : 1.0;
+	color.red   = (int) (cp->_priv->r * 65535 + 0.5);
+	color.green = (int) (cp->_priv->g * 65535 + 0.5);
+	color.blue  = (int) (cp->_priv->b * 65535 + 0.5);
+	alpha       = cp->_priv->use_alpha ? (int) (cp->_priv->a * 65535 + 0.5) : 65535;
 
-	/* Hack: we set the color twice so that GtkColorSelection will remember its history */
-	gtk_color_selection_set_color (GTK_COLOR_SELECTION (csd->colorsel), color);
-	gtk_color_selection_set_color (GTK_COLOR_SELECTION (csd->colorsel), color);
+	gtk_color_selection_set_previous_color (GTK_COLOR_SELECTION (csd->colorsel), &color);
+	gtk_color_selection_set_previous_alpha (GTK_COLOR_SELECTION (csd->colorsel), alpha);
+
+	gtk_color_selection_set_current_color (GTK_COLOR_SELECTION (csd->colorsel), &color);
+	gtk_color_selection_set_current_alpha (GTK_COLOR_SELECTION (csd->colorsel), alpha);
 
 	gtk_widget_show (cp->_priv->cs_dialog);
 }
@@ -714,7 +741,7 @@ gnome_color_picker_set_d (GnomeColorPicker *cp, gdouble r, gdouble g, gdouble b,
 	cp->_priv->a = a;
 
 	render (cp);
-	gtk_widget_draw (cp->_priv->drawing_area, NULL);
+	gtk_widget_queue_draw (cp->_priv->drawing_area);
 }
 
 
@@ -775,7 +802,7 @@ gnome_color_picker_set_i8 (GnomeColorPicker *cp, guint8 r, guint8 g, guint8 b, g
 	cp->_priv->a = a / 255.0;
 
 	render (cp);
-	gtk_widget_draw (cp->_priv->drawing_area, NULL);
+	gtk_widget_queue_draw (cp->_priv->drawing_area);
 }
 
 
@@ -836,7 +863,7 @@ gnome_color_picker_set_i16 (GnomeColorPicker *cp, gushort r, gushort g, gushort
 	cp->_priv->a = a / 65535.0;
 
 	render (cp);
-	gtk_widget_draw (cp->_priv->drawing_area, NULL);
+	gtk_widget_queue_draw (cp->_priv->drawing_area);
 }
 
 
@@ -891,7 +918,7 @@ gnome_color_picker_set_dither (GnomeColorPicker *cp, gboolean dither)
 	cp->_priv->dither = dither ? TRUE : FALSE;
 
 	render (cp);
-	gtk_widget_draw (cp->_priv->drawing_area, NULL);
+	gtk_widget_queue_draw (cp->_priv->drawing_area);
 }
 
 
@@ -934,7 +961,7 @@ gnome_color_picker_set_use_alpha (GnomeColorPicker *cp, gboolean use_alpha)
 	cp->_priv->use_alpha = use_alpha ? TRUE : FALSE;
 
 	render (cp);
-	gtk_widget_draw (cp->_priv->drawing_area, NULL);
+	gtk_widget_queue_draw (cp->_priv->drawing_area);
 }
 
 /**
